;
;Diagnosticky znalostni system SP410PRO
;Tento system je primarne urceny pro opravy srovnavaci a tloustkovaci
;frezky Jerabek SP410PRO. Ovsem je mozne jej pouzit i pro obdobne
;stroje urcene pro mala a stredni truhlarstvi.
;
;Soucast bakalarske prace zvane "Vyuka reprezentace znalosti"
;
;Autor: Josef Kazak A11B0397P
;
;Datum: 27. 4. 2015
;

;Zde se nachazi komunikace s uzivatelem, ktera slouzi ke zjisteni, zda dana problemova
;frezka je primo Jerabek SP410PRO nebo alespon zda spada do stejne kategorie.
;Pote je jeste zjistovano, zda je stroj vybaven elektromagnetickou brzdou, pokud to
;tedy neni frezka od Jerabka, v tom pripade se otazka samozrejme nepoklada.

;Pravidlo vypisuje uvitani (vcetne navodu na ovladani znalostniho systemu),
;pote ziskava informaci, zda chce uzivatel provest zaznam identifikace problemu.
(defrule nove-spusteni
 (start-programu)
 (not (dotaz-kategorie ?))
=>
 (printout t crlf crlf crlf crlf
 "           *** Vitejte v diagnostickem znalostnim systemu! ***" crlf crlf
 "      ****************************************" crlf
 "      Tento system je primarne urceny pro opravy srovnavaci a tloustkovaci" crlf
 "      frezky Jerabek SP410PRO. Ovsem je mozne jej pouzit i pro obdobne" crlf
 "      stroje urcene pro mala a stredni truhlarstvi." crlf crlf
 "      Ovladani systemu:"crlf
 "      Na konci kazde otazky je mnozina odpovedi. Vzdy vlozte jednu z nich" crlf
 "      a potvrdte klavesou Enter." crlf
 "      ****************************************"
 )

 (printout t crlf crlf crlf crlf "      Prejete si zaznamenat prubeh identifikace problemu?")
 (bind ?zaznamenavat (ziskej-odpoved ano ne ukoncit))
 (if (eq ?zaznamenavat ano) then
  (bind ?nazev-souboru (ziskej-nazev-souboru))
  (bind ?vyskyt-chyby ano)
  (if (neq ?nazev-souboru chyba) then
   (if (dribble-on ?nazev-souboru) then
	(printout t crlf 
	"      Diagnosticky znalostni system zaznamenava prubeh identifikace" crlf 
	"      problemu do souboru \"" ?nazev-souboru "\".")
    (assert (zaznam-logu))
	(bind ?vyskyt-chyby ne)
   )
  )

  (if (eq ?vyskyt-chyby ano) then
   (printout t crlf
   "      Neni mozne zapisovat do souboru, bud nemate opravneni" crlf
   "      nebo je pevny disk plny.")
  )

  (pokracovat)
 )

 (assert (zjisti-kategorii))
)

;Pravidlo poklada dotaz, o jakou frezku se jedna. Jestli o Jerabek SP410PRO
;nebo jestli se alespon jedna o frezku ze stejne kategorie.
(defrule otazka-typ-frezky
 (start-programu)
 ?kategorie <- (zjisti-kategorii)
=>
 (retract ?kategorie)
 (printout t crlf crlf crlf crlf
 "      V pripade Vasi srovnavaci a tloustkovaci frezky se jedna o stroj" crlf	
 "      Jerabek SP410PRO nebo je dana frezka alespon ze stejne kategorie" crlf
 "      (tedy je urcena pro male a stredni provozy)?")
 (assert (dotaz-kategorie (ziskej-odpoved ano ne nevim ukoncit)))
)

;Pokud uzivatel vlastni frezku Jerabek SP410PRO, pak jiz neni potreba dalsi
;identifikace stroje. Baze faktu je tedy upravena tak, ze se ihned prechazi
;k pravidlu "slozeni-faktu-a-povoleni-identifikace-problemu", ktere je
;vstupnim bodem k identifikaci problemu.
(defrule typ-frezky-odpoved-ano
 (start-programu)
 ?dotaz <- (dotaz-kategorie ano)
=>
 (retract ?dotaz)
 (assert (existuje-elmag-brzda ano) (stroj-odpovida ano))
)

;Pokud uzivatel vlastni jinou frezku (i z jine kategorie), pak je dotazan,
;zda chce program ukoncit, nebo zmenit sve posledni rozhodnuti.
(defrule typ-frezky-odpoved-ne
 (start-programu)
 ?dotaz <- (dotaz-kategorie ne)
=>
 (retract ?dotaz)
 (printout t crlf crlf crlf crlf
 "      Tento znalostni system je primarne urceny pro kombinovanou frezku" crlf
 "      Jerabek SP410PRO a podobnym strojum ze stejne kategorie. Chcete" crlf
 "      zmenit svou odpoved, nebo ukoncit program?")
 (assert (program-ukoncit-otazka (ziskej-odpoved zmenit ukoncit)))
)

;Uprava baze faktu, pokud uzivatel chce zmenit sve posledni rozhodnuti.
(defrule typ-frezky-odpoved-ne-zmenit-odpoved
 (start-programu)
 ?dotaz <- (program-ukoncit-otazka zmenit)
=>
 (retract ?dotaz)
 (assert (zjisti-kategorii))
)

;Vypis charakteristickych znaku srovnavacich a tloustkovacich frezek,
;urcenych pro mala a stredni truhlarstvi.
;K tomuto pravidlu se uzivatel dostane odpovedi "nevim", v otazce, jakou
;frezku vlastni.
(defrule typ-frezky-odpoved-nevim
 (start-programu)
 ?dotaz <- (dotaz-kategorie nevim)
=>
 (retract ?dotaz)
 (printout t crlf crlf crlf crlf
 "      Kategorie srovnavacich a tloustkovacich frezek pro mala a stredni" crlf
 "      truhlarstvi se vyznacuji nasledujicimi rysy:" crlf
 "        - vsechny funkce pohani pouze 1 motor" crlf
 "        - jedna nozova hridel (obvykle se 3-4 nozi)" crlf
 "        - manualni nastaveni tloustky obrabeni" crlf
 "        - manualni nastaveni pravitka srovnavani" crlf
 "        - jedna se o tzv. \"kombinaci\", tedy srovnavaci a tloustkovaci" crlf
 "          frezka v ramci jedineho stroje" crlf crlf
 "      Splnuje Vase frezka vsechny vyse uvedene body?")
 (assert (souhlasi-kategorie (ziskej-odpoved ano ne ukoncit)))
)

;Pravidlo vypisuje otazku, zda chce uzivatel program ukoncit, nebo zmenit
;sva rozhodnuti.
;
(defrule typ-frezky-odpoved-nevim-odpoved-ne
 (start-programu)
 ?dotaz <- (souhlasi-kategorie ne)
=>
 (retract ?dotaz)
 (printout t crlf crlf crlf crlf
 "      Pro Vasi frezku neni tento znalostni system urcen. Muzete sve" crlf
 "      odpovedi pozmenit nebo program ukoncit: ")
 (assert (program-ukoncit-otazka (ziskej-odpoved zmenit ukoncit)))
)

;Pokud dana problemova frezka patri do stejne kategorie jako Jerabek SP410PRO,
;tak je polozena otazka, zjistujici, zda ma dana frezka elektromagnetickou brzdu.
(defrule typ-frezky-odpoved-nevim-existuje-brzda
 (start-programu)
 ?dotaz <- (souhlasi-kategorie ano)
=>
 (retract ?dotaz)
 (printout t crlf crlf
 "      Muzete tedy tento znalostni system vyuzivat pro opravy zavad" crlf
 "      Vasi frezky, ovsem vysledne rady (a doporuceni) berte s ohledem" crlf
 "      k Vasi frezce a vzdy nejprve dukladne prostudujte manual" crlf
 "      dane frezky." crlf)
 (pokracovat)
 (printout t crlf crlf crlf crlf "      Je Vase frezka vybavena elektromagnetickou brzdou?")
 (assert (existuje-elmag-brzda (ziskej-odpoved ano ne nevim ukoncit)) (stroj-odpovida ano))
)

;Pravidlo podava vysvetleni, jak se pozna, ze dana problemova frezka ma
;elektromagnetickou brzdu.
(defrule typ-frezky-odpoved-nevim-existuje-brzda-odpoved-nevim
 (start-programu)
 ?existuje-brzda <- (existuje-elmag-brzda nevim)
=>
 (retract ?existuje-brzda)
 (printout t crlf crlf crlf crlf
 "      Pritomnost brzdy lze poznat nekolika zpusoby. Napriklad prostudovanim manualu" crlf 
 "      dane frezky. Ci poslechem, zda stroj po vypnuti vydava hluk (z oblasti motoru)," crlf 
 "      i kdyz se nozova hridel jiz netoci. Existuje tedy elektromagneticka brzda v dane" crlf
 "      frezce?")
 (assert (zvuk-elmag-brzda (ziskej-odpoved ano ne ukoncit)))
)

;Vypis na obrazovku ohledne existence/neexistence elektromagneticke brzdy.
(defrule typ-frezky-odpoved-nevim-existuje-brzda-odpoved-nevim-oznameni
 (start-programu)
 ?dotaz-zvuk <- (zvuk-elmag-brzda ?pom&ano|ne)
=>
 (retract ?dotaz-zvuk)
 (assert (existuje-elmag-brzda ?pom))
 
 (bind ?dispozice nedisponuje)
 (if (eq ?pom ano)
  then
   (bind ?dispozice disponuje) 
 )

 (printout t crlf crlf
 "      Vase frezka " ?dispozice " elektromagnetickou brzdou. Tento fakt" crlf
 "      bude pri rozhodovani zohlednen." crlf)
 (pokracovat)
)

;Jednotne prechodove pravidlo. Pokud jsou ziskany vsechny dulezite
;informace, tak je upravena baze faktu tak, aby mozne pristoupit
;k identifikaci problemu.
(defrule slozeni-faktu-a-povoleni-identifikace-problemu
 (declare (salience 30))
 ?start <- (start-programu)
 ?brzda <- (existuje-elmag-brzda ?brzda-existuje&ano|ne)
 ?stroj <- (stroj-odpovida ?kategorie-stroje-odpovida&ano)
=>
 (retract ?start ?brzda ?stroj)
 (assert (aktualni-uroven) (frezka (odpovida ?kategorie-stroje-odpovida) (elmag-brzda ?brzda-existuje)))
)
