;
;Diagnosticky znalostni system SP410PRO
;Tento system je primarne urceny pro opravy srovnavaci a tloustkovaci
;frezky Jerabek SP410PRO. Ovsem je mozne jej pouzit i pro obdobne
;stroje urcene pro mala a stredni truhlarstvi.
;
;Soucast bakalarske prace zvane "Vyuka reprezentace znalosti"
;
;Autor: Josef Kazak A11B0397P
;
;Datum: 27. 4. 2015
;

;V tomto souboru jsou ulozena hlavne pravidla, zajistujici pohyb 
;v hierarchii otazek. Tyto otazky slouzi pro identifikaci a urceni
;reseni daneho problemu. Dale jsou zde ruzne funkce (urcene napr. 
;pro spravne nacteni uzivatelskych vstupu) a neposledni v rade
;se v tomto souboru nachazeji sablony, slouzici pro spravny vyber
;pravidel a chod tohoto systemu.

;*********************************************************
;Oddil sablon slouzicich napr. k popisu prubehu identifikace problemu 
;ci k udrzovani informace o aktualni urovni zanoreni ve stromu.


;Sablona "hypoteza" je behem exekuce velmi casto vyuzivana. Jelikoz uklada 
;informaci o prubezne identifikaci problemu, tato informace je klicova ke 
;spousteni identifikacnich pravidel. Dale pak sablona obsahuje "ukazatel", 
;ze byl vydan poznatek a slot se samotnym textem poznatku.
;Sloty:
;  - popis - tento slot je behem identifikace problemu rozsirovan o nova
;            pole, ktera pridavaji jednotliva identifikacni pravidla. A naopak
;            je tento "vicepolovy" slot zkracovan, kdyz uzivatel zada "zpet"
;            behem identifikace.
;  - identifikovano - nabyva hodnoty "ano", pokud byla vydana odpoved
;  - poznatek - text poznatku
(deftemplate hypoteza
 (multislot popis (type SYMBOL) (default))
 (slot identifikovano (type SYMBOL) (default ne))
 (multislot poznatek (type STRING) (default ""))
)

;Sablona "aktualni-uroven" udrzuje informaci o aktualni urovni zanoreni
;ve stromu identifikace problemu.
;Sloty:
;  - hodnota - numericke oznaceni urovne zanoreni
(deftemplate aktualni-uroven
 (slot hodnota (type INTEGER) (default 1))
)

;Tato sablona prenasi informaci o tom, zda je nutny odborny servis 
;(nebo elektrikar) k naprave daneho problemu.
;Sloty:
;  - elektrikar - znaci, ze dany problem by mel vyresit elektrikar
;  - servis - dany problem by mel resit odborny servis
(deftemplate odborna-pomoc
 (slot elektrikar (allowed-values ano ne) (default ne))
 (slot servis (allowed-values ano ne) (default ne))
)

;Sablona disponuje informacemi o dane problemove frezce, zda se jedna 
;primo o frezku Jerabek SP410PRO (nebo alespon odpovida dane kategorii) 
;a zda je vybavena elektromagnetickou brzdou. 
;Sloty:
;  - odpovida - hodnota je ano, pokud je dana problemova frezka primo 
;               Jerabek SP410PRO (nebo alespon spada do dane kategorie)
;  - elmag-brzda - pokud je problemova frezka vybavena elektromagnetickou
;                  brzdou, pak je hodnota "ano", v opacnem pripade "ne"
(deftemplate frezka
 (slot odpovida (type SYMBOL) (allowed-values ano ne))
 (slot elmag-brzda (type SYMBOL) (allowed-values ano ne))
)

;*********************************************************
;Oddil funkci programu, slouzi napr. pro ziskani bezchybneho uzivatelskeho vstupu.

;Na zaklade mnoziny moznych odpovedi, ziskanych v parametrech metody, je vytvoren 
;retezec, vyzyvajici uzivatele k zadani odpovedi na danou otazku. Pote je retezec 
;opakovane vypisovan na obrazovku, dokud uzivatel nezada spravne jednu z moznych 
;odpovedi.
;Parametry:
;  - $?moznosti - znaci mnozinu moznych odpovedi na danou otazku. Pokud neni zadana
;                 zadna mozna odpoved, pak je program okamzite ukoncen.
(deffunction ziskej-odpoved ($?moznosti)
 (bind ?konec-cyklu ne)
 (bind ?delka-mnoziny (length $?moznosti))
 (bind ?delka-minus-jedna (- ?delka-mnoziny 1))
 (bind ?vysledek)
 
 ;Pokud je vice nez jeden parametr, pak dojde ke slozeni a vypisu vyzvy. V opacnem 
 ;pripade je program ukoncen.
 (if (> ?delka-mnoziny 0) then
  ;Slozeni vypisu moznych odpovedi.
  (bind ?text "      Zadejte jednu z moznosti [")
  (loop-for-count (?i 1 ?delka-minus-jedna) do
   (bind ?text (str-cat ?text (format nil "%s/" (nth$ ?i $?moznosti))))
  )
  (bind ?text (str-cat ?text (format nil "%s]: " (nth$ ?delka-mnoziny $?moznosti))))

  ;Kontrola uzivatelskeho vstupu.
  (while (neq ?konec-cyklu ano) do
   (printout t crlf ?text)
   (bind ?vysledek (read))
  
   (if (lexemep ?vysledek) then
    (bind ?vysledek (lowcase ?vysledek)) 
   )

   (loop-for-count (?index 1 ?delka-mnoziny) do
    (if (eq ?vysledek (nth$ ?index $?moznosti)) then
     (bind ?konec-cyklu ano)
	  
	 (if (eq ?vysledek ukoncit) then
	  (assert (ukoncit-program))
	 )

	 (break)
    )
   )
  )
  else
   (bind ?vysledek ukoncit)
   (assert (ukoncit-program))
 )
 
 (return ?vysledek)
)

;Procedura vyzyva ke stisknuti klavesy Enter. Tento podprogram slouzi k "pozastaveni"
;behu programu, kdyz je napr. na obrazovku vypsan poznatek.
(deffunction pokracovat ()
 (printout t crlf "      Pro pokracovani stisknete klavesu Enter.")
 (bind ?pokracovat (readline))
)

;Funkce je pouzita, pokud uzivatel chce, aby byl zaznamenan prubeh identifikace
;problemu do souboru.
;Pokud existuje soubor "data.csv", pak podprogram z nej precte poradove cislo 
;posledniho zaznamu identifikace. To zvysi o 1 a zapise zpet do souboru. Nakonec 
;vrati odpoved (nazev souboru, do ktereho se bude prubeh ukladat) s upravenym 
;poradovnym cislem. 
;V pripade, ze dany soubor "data.csv" neexistuje, pak by mel byt vytvoren se 
;zapsanou 0 (ktera je pak take soucasti odpovedi). Pokud dany soubor nelze 
;vytvorit (plny disk, nejsou prirazena pravat atd.), pak je nahlasena chyba.
(deffunction ziskej-nazev-souboru ()
 (bind ?cislo -1)
 (if (open "data.csv" soubor "r") then
   (bind ?cislo (read soubor))
   (close soubor)
 )

 (if (open "data.csv" soubor "w") then
  (if (and (integerp ?cislo) (> ?cislo -1)) then
   (bind ?cislo (+ ?cislo 1))
  else
   (bind ?cislo 0)
  )
  (printout soubor ?cislo)
  (close soubor)
  
  (return (str-cat "soubor" ?cislo ".txt"))
 else
  (return chyba)
 )
) 

;*********************************************************
;Oddil s pravidly pro pocatecni upravu baze faktu a pro radne 
;ukonceni znalostniho systemu.

;Inicializace programu.
(defrule inicializace
 (declare (salience 90))
 ?x <- (initial-fact) 
=>
 (retract ?x)
 (assert (start-programu))
)

;Pravidlo slouzi pro radne ukonceni. Pokud je prubeh identifikace zaznamenavan,
;pak je tato cinnost ukoncena.
(defrule ukoncit-zaznam-logu
 (declare (salience 30))
 (ukoncit-program)
 (zaznam-logu)
=>
 (dribble-off)
)

;Pravidlo slouzi pro radne ukonceni systemu. Resetuje bazi faktu a pote do ni 
;nacita pocatecni fakty. Nakonec dojde k ukonceni behu.
(defrule ukoncit-program
 (declare (salience 30))
 (ukoncit-program)
=>
 (printout t crlf crlf crlf crlf "      Ukoncuji program!" crlf)
 (reset)
 (load-facts pocatecni-fakta.fct)
 (halt)
)

;*********************************************************
;Pravidla pracujici s fakty. Slouzi napr. k pohybu v hierarchii identifikace problemu.

;Pravidlo se spusti, pokud si uzivatel nevybere ani jednu tezi/kategorii z nabizenych. Pravidlo
;jej vyzyva, zda chce program ukoncit, zobrazit teze znovu nebo vratit posledni rozhodnuti zpet
;(pokud tedy neni uroven zanoreni 1).
(defrule iden-uroven-?-nevybrano
 (declare (salience -30))
 (aktualni-uroven (hodnota ?uroven))
 (hypoteza (identifikovano ne))
 (nezadouci ?)
=>
 (if (> ?uroven 1) then
  (printout t crlf crlf crlf crlf
  "      Nevybral jste si ani jednu kategorii. Chcete projit vyber" crlf
  "      znovu, vratit posledni rozhodnuti nebo ukoncit program?")
  (assert (odpoved-identifikace (ziskej-odpoved znovu zpet ukoncit) nevybrano))
 else
  (printout t crlf crlf crlf crlf
  "      Nevybral jste si ani jednu z moznych kategorii. Chcete projit" crlf
  "      vyber znovu nebo ukoncit program?")
  (assert (odpoved-identifikace (ziskej-odpoved znovu ukoncit) nevybrano))
 )
)

;Pravidlo upravuje bazi faktu tak, ze presune uzivatele hloubeji v danem podstromu 
;identifikace problemu.
;Je vyuzito, kdyz uzivatel odpovi "ano" na nekterou tezi/kategorii.
(defrule posun-na-nizsi-uroven-zanoreni
 (declare (salience 30))
 (aktualni-uroven (hodnota ?))
 ?iden-problemu <- (odpoved-identifikace ano ?pom)
 ?teze <- (hypoteza (popis $?prubeh))
 (not (nezadouci ?))
=>
 (retract ?iden-problemu)
 (assert (posun-uroven +))
 (modify ?teze (popis $?prubeh ?pom)) 
)

;Pravidlo upravuje bazi faktu tak, ze vrati posledni rozhodnuti uzivatele a tim
;posune uzivatele v danem podstromu (identifikace problemu) o uroven vyse.
;Je vyuzito, kdyz uzivatel odpovi "zpet" na nekterou tezi/kategorii.
(defrule posun-na-vyssi-uroven-zanoreni
 (declare (salience 30))
 ?iden-problemu <- (odpoved-identifikace zpet ?)
 ?teze <- (hypoteza (popis $?prubeh ?))
 (aktualni-uroven (hodnota ?))
 (not (nezadouci ?))
=>
 (retract ?iden-problemu)
 (assert (posun-uroven -))
 (modify ?teze (popis $?prubeh))
)

;Pravidlo se spousti pri jakekoliv uzivatelove odpovedi na otazku,
;krome odpovedi "zpet". Pravidlo upravi bazi faktu tak, ze posune
;uzivatele v danem podstromu nize.
(defrule posun-na-nizsi-uroven-podle-rozhodnuti
 ?odpoved <- (odpoved-rozhodnuti ?rozhodnuti&~zpet ?pom)
 ?teze <- (hypoteza (popis $?prubeh))
=>
 (retract ?odpoved)
 (modify ?teze (popis $?prubeh ?pom ?rozhodnuti))
 (assert (posun-uroven +))
)

;Pokud uzivatel zada "zpet", pricemz ma odpovedet na otazku a predchozi akce
;bylo take rozhodnuti, pak je baze faktu upravena tak, aby posunu uzivatele
;zpet v hierarchii identifikace problemu.
(defrule posun-na-vyssi-uroven-podle-rozhodnuti-predchudce-je-rozhodnuti
 ?odpoved <- (odpoved-rozhodnuti zpet ?)
 ?teze <- (hypoteza (popis $?prubeh ?cast-a ?cast-b&ano|ne|nevim))
=>
 (retract ?odpoved)
 (modify ?teze (popis $?prubeh))
 (assert (posun-uroven -))
)

;Pokud uzivatel zada "zpet", pricemz ma odpovedet na otazku a predchozi akce
;byl vyber teze/kategorie, pak je baze faktu upravena tak, aby posunu uzivatele
;zpet v hierarchii identifikace problemu.
(defrule posun-na-vyssi-uroven-podle-rozhodnuti-predchudne-neni-rozhodnuti
 ?odpoved <- (odpoved-rozhodnuti zpet ?)
 ?teze <- (hypoteza (popis $?prubeh ?posledni-rozhodnuti&~ano|~ne|~nevim))
=>
 (retract ?odpoved)
 (modify ?teze (popis $?prubeh))
 (assert (posun-uroven -))
)

;Pravidlo inkrementuje/dekrementuje hodnotu (popisujici aktualni
;uroven zanoreni v danem podstromu) o 1;
(defrule zmen-aktualni-uroven
 ?posun <- (posun-uroven ?operator)
 ?uroven <- (aktualni-uroven (hodnota ?uroven-hodnota))
=>
 (retract ?posun)
 (if (eq ?operator +) then
  (bind ?uroven-hodnota (+ ?uroven-hodnota 1))
  else
  (bind ?uroven-hodnota (- ?uroven-hodnota 1))
 )
 (modify ?uroven (hodnota ?uroven-hodnota))
)

;Pravidlo odstranuje veskere fakty, jejichz nazev je "nezadouci". Je
;"spousteno" pri jakemkoliv prechodu v hierarchii a muze byt aplikovano 
;(behem jednoho prechodu) i opakovane.
(defrule odstraneni-nezadouci-skupiny-pri-posunu
 (declare (salience 35))
 (aktualni-uroven (hodnota ?))
 (odpoved-identifikace ano|znovu|zpet ?)
 ?nezadouci-fakt <- (nezadouci ?)
=>
 (retract ?nezadouci-fakt)
)

;Pokud uzivatel odpovi "ne" (na aktualni tezi), pak je pridan fakt
;("nezadouci") do baze, coz zarucuje, ze se dane pravidlo (v dane 
;"relaci") neaplikuje znovu.
(defrule zamitnuti-daneho-vyberu
 (declare (salience 30))
 (aktualni-uroven (hodnota ?))
 ?odpoved <- (odpoved-identifikace ne ?pom)
=>
 (retract ?odpoved)
 (assert (nezadouci ?pom))
)

;Pokud uzivatel zada "znovu" (na otazku, ktera se nachazi v pravidle
;"iden-uroven-?-nevybrano"), tak je upravena baze tak, aby se jiz
;pouzita pravidla (nabizejici teze/kategorie) zopakovala.
(defrule zobrazeni-aktualni-urovne-dotazu-znovu
 (declare (salience 30))
 ?odpoved-znovu <- (odpoved-identifikace znovu nevybrano)
 ?uroven <- (aktualni-uroven (hodnota ?pom))
=>
 (retract ?odpoved-znovu)
 (modify ?uroven (hodnota ?pom))
)

;*********************************************************
;Oddil vypisu na obrazovku.

;Pravidlo vypisuje text poznatku na obrazovku.
(defrule vypis-poznatku
 (declare (salience 60))
 (hypoteza (identifikovano ano) (poznatek $?text-doporuceni))
 ?uroven <- (aktualni-uroven (hodnota ?))
 (odborna-pomoc (elektrikar ?nutny-elektri) (servis ?nutny-servis))
=>
 (retract ?uroven)
 (bind ?pocet-radku (length $?text-doporuceni))
 (printout t crlf crlf crlf crlf "      DOPORUCENI: " (nth$ 1 $?text-doporuceni))
 (loop-for-count (?index 2 ?pocet-radku) do
  (printout t crlf (format nil "                  %s" (nth$ ?index $?text-doporuceni)))
 )
 (if (and (eq ne ?nutny-elektri) (eq ne ?nutny-servis)) then
  (pokracovat)
 )
)

;Dodatek k vypisu poznatku, rikajici, ze dany problem by mel resit elektrikar.
(defrule potreba-pomoci-elektrikare
 (declare (salience 60))
 ?pomoc <- (odborna-pomoc (elektrikar ano))
 (hypoteza (identifikovano ano))
=>
 (modify ?pomoc (elektrikar ne))
 (printout t crlf "      UPOZORNENI: Tento ukon by mela provadet osoba s patricnym opravnenim!")
 (pokracovat)
)

;Dodatek k vypisu poznatku, rikajici, ze dany problem by mel resit odborny servis.
(defrule potreba-pomoci-servisu
 (declare (salience 60))
 ?pomoc <- (odborna-pomoc (servis ano))
 (hypoteza (identifikovano ano))
=>
 (modify ?pomoc (servis ne))
 (printout t crlf crlf
 "      UPOZORNENI: Pro vyreseni vaseho problemu je nejlepsi kontaktovat odborny servis." crlf
 "                  Jelikoz zavada muze byt rozsahlejsiho charakteru (pripadne se" crlf
 "                  nepodarilo presne identifikovat dany problem).")
 (pokracovat)
)
 
;Po kompletni vypsani poznatku na obrazovku, nasleduje otazka, zda se ma znalostni
;system ukoncit, nebo spustit znovu.
(defrule spustit-program-znovu-otazka
 (declare (salience 30))
 (hypoteza (identifikovano ano))
 (not (aktualni-uroven))
=>
 (printout t crlf crlf crlf crlf "      Spustit program znovu, nebo ukoncit?")
 (assert (spustit-znovu-otazka (ziskej-odpoved znovu ukoncit)))
)

;Uprava baze faktu, pokud chce uzivatel spustit znalostni system znovu (po vypisu
;poznatku).
(defrule spustit-znovu
 (declare (salience 30))
 ?dotaz <- (spustit-znovu-otazka znovu)
 ?teze <- (hypoteza (identifikovano ano))
 ?pomoc <- (odborna-pomoc)
=>
 (retract ?dotaz ?teze ?pomoc)
 (assert (aktualni-uroven) (hypoteza) (odborna-pomoc))
)
